@@ -738,8 +738,8 @@ show_dpif(struct dpif *dpif, struct dpctl_params *dpctl_p)
                 continue;
             }
             error = netdev_get_stats(netdev, &s);
+            netdev_close(netdev);
             if (!error) {
-                netdev_close(netdev);
                 print_stat(dpctl_p, "    RX packets:", s.rx_packets);
                 print_stat(dpctl_p, " errors:", s.rx_errors);
                 print_stat(dpctl_p, " dropped:", s.rx_dropped);
@@ -1359,19 +1359,17 @@ static int
 dpctl_del_flow_dpif(struct dpif *dpif, const char *key_s,
                     struct dpctl_params *dpctl_p)
 {
-    struct dpif_flow_stats stats;
-    struct dpif_port dpif_port;
     struct dpif_port_dump port_dump;
-    struct ofpbuf key;
-    struct ofpbuf mask; /* To be ignored. */
-
-    ovs_u128 ufid;
-    bool ufid_generated;
-    bool ufid_present;
+    struct dpif_flow_stats stats;
+    bool ufid_generated = false;
+    struct dpif_port dpif_port;
+    bool ufid_present = false;
     struct simap port_names;
+    struct ofpbuf mask; /* To be ignored. */
+    struct ofpbuf key;
+    ovs_u128 ufid;
     int n, error;
 
-    ufid_present = false;
     n = odp_ufid_from_string(key_s, &ufid);
     if (n < 0) {
         dpctl_error(dpctl_p, -n, "parsing flow ufid");
