@@ -26,6 +26,7 @@
 #include "openvswitch/json.h"
 #include "openvswitch/list.h"
 #include "openvswitch/ofpbuf.h"
+#include "ovs-replay.h"
 #include "ovs-thread.h"
 #include "openvswitch/poll-loop.h"
 #include "reconnect.h"
@@ -220,19 +221,19 @@ jsonrpc_log_msg(const struct jsonrpc *rpc, const char *title,
         }
         if (msg->params) {
             ds_put_cstr(&s, ", params=");
-            json_to_ds(msg->params, 0, &s);
+            json_to_ds(msg->params, JSSF_SORT, &s);
         }
         if (msg->result) {
             ds_put_cstr(&s, ", result=");
-            json_to_ds(msg->result, 0, &s);
+            json_to_ds(msg->result, JSSF_SORT, &s);
         }
         if (msg->error) {
             ds_put_cstr(&s, ", error=");
-            json_to_ds(msg->error, 0, &s);
+            json_to_ds(msg->error, JSSF_SORT, &s);
         }
         if (msg->id) {
             ds_put_cstr(&s, ", id=");
-            json_to_ds(msg->id, 0, &s);
+            json_to_ds(msg->id, JSSF_SORT, &s);
         }
         VLOG_DBG("%s: %s %s%s", rpc->name, title,
                  jsonrpc_msg_type_to_string(msg->type), ds_cstr(&s));
@@ -888,7 +889,7 @@ jsonrpc_session_open_multiple(const struct svec *remotes, bool retry)
         reconnect_set_backoff(s->reconnect, INT_MAX, INT_MAX);
     }
 
-    if (!stream_or_pstream_needs_probes(name)) {
+    if (!stream_or_pstream_needs_probes(name) || ovs_replay_is_active()) {
         reconnect_set_probe_interval(s->reconnect, 0);
     }
 
@@ -914,6 +915,11 @@ jsonrpc_session_open_unreliably(struct jsonrpc *jsonrpc, uint8_t dscp)
     reconnect_set_name(s->reconnect, jsonrpc_get_name(jsonrpc));
     reconnect_set_max_tries(s->reconnect, 0);
     reconnect_connected(s->reconnect, time_msec());
+
+    if (ovs_replay_is_active()) {
+        reconnect_set_probe_interval(s->reconnect, 0);
+    }
+
     s->dscp = dscp;
     s->rpc = jsonrpc;
     s->stream = NULL;
@@ -946,6 +952,19 @@ jsonrpc_session_steal(struct jsonrpc_session *s)
     return rpc;
 }
 
+void
+jsonrpc_session_replace(struct jsonrpc_session *s, struct jsonrpc *rpc)
+{
+    if (s->rpc) {
+        jsonrpc_close(s->rpc);
+    }
+    s->rpc = rpc;
+    if (s->rpc) {
+        reconnect_set_name(s->reconnect, jsonrpc_get_name(s->rpc));
+        reconnect_connected(s->reconnect, time_msec());
+    }
+}
+
 static void
 jsonrpc_session_disconnect(struct jsonrpc_session *s)
 {
@@ -1297,6 +1316,9 @@ void
 jsonrpc_session_set_probe_interval(struct jsonrpc_session *s,
                                    int probe_interval)
 {
+    if (ovs_replay_is_active()) {
+        return;
+    }
     reconnect_set_probe_interval(s->reconnect, probe_interval);
 }
 
@@ -1315,6 +1337,15 @@ jsonrpc_session_set_dscp(struct jsonrpc_session *s, uint8_t dscp)
     }
 }
 
+void
+jsonrpc_session_set_options(struct jsonrpc_session *s,
+                            const struct jsonrpc_session_options *options)
+{
+    jsonrpc_session_set_max_backoff(s, options->max_backoff);
+    jsonrpc_session_set_probe_interval(s, options->probe_interval);
+    jsonrpc_session_set_dscp(s, options->dscp);
+}
+
 /* Sets thresholds for send backlog.  If send backlog contains more than
  * 'max_n_msgs' messages or is larger than 'max_backlog_bytes' bytes,
  * connection will be closed (then reconnected, if that feature is enabled). */
