@@ -669,6 +669,7 @@ netdev_bsd_rxq_drain(struct netdev_rxq *rxq_)
     struct ifreq ifr;
     struct netdev_rxq_bsd *rxq = netdev_rxq_bsd_cast(rxq_);
 
+    memset(&ifr, 0, sizeof ifr);
     strcpy(ifr.ifr_name, netdev_get_kernel_name(netdev_rxq_get_netdev(rxq_)));
     if (ioctl(rxq->fd, BIOCFLUSH, &ifr) == -1) {
         VLOG_DBG_RL(&rl, "%s: ioctl(BIOCFLUSH) failed: %s",
@@ -828,6 +829,7 @@ netdev_bsd_get_mtu(const struct netdev *netdev_, int *mtup)
     if (!(netdev->cache_valid & VALID_MTU)) {
         struct ifreq ifr;
 
+        memset(&ifr, 0, sizeof ifr);
         error = af_inet_ifreq_ioctl(netdev_get_kernel_name(netdev_), &ifr,
                                     SIOCGIFMTU, "SIOCGIFMTU");
         if (!error) {
@@ -1168,6 +1170,27 @@ cleanup:
     return error;
 }
 
+static int
+netdev_bsd_get_speed(const struct netdev *netdev, uint32_t *current,
+                     uint32_t *max)
+{
+    enum netdev_features f_current, f_supported, f_advertised, f_peer;
+    int error;
+
+    error = netdev_bsd_get_features(netdev, &f_current, &f_advertised,
+                                    &f_supported, &f_peer);
+    if (error) {
+        return error;
+    }
+
+    *current = MIN(UINT32_MAX,
+                   netdev_features_to_bps(f_current, 0) / 1000000ULL);
+    *max = MIN(UINT32_MAX,
+               netdev_features_to_bps(f_supported, 0) / 1000000ULL);
+
+    return 0;
+}
+
 /*
  * Assigns 'addr' as 'netdev''s IPv4 address and 'mask' as its netmask.  If
  * 'addr' is INADDR_ANY, 'netdev''s IPv4 address is cleared.  Returns a
@@ -1419,6 +1442,8 @@ do_set_addr(struct netdev *netdev,
             struct in_addr addr)
 {
     struct ifreq ifr;
+
+    memset(&ifr, 0, sizeof ifr);
     make_in4_sockaddr(&ifr.ifr_addr, addr);
     return af_inet_ifreq_ioctl(netdev_get_kernel_name(netdev), &ifr, ioctl_nr,
                                ioctl_name);
@@ -1493,6 +1518,7 @@ netdev_bsd_update_flags(struct netdev *netdev_, enum netdev_flags off,
     .get_carrier = netdev_bsd_get_carrier,           \
     .get_stats = netdev_bsd_get_stats,               \
     .get_features = netdev_bsd_get_features,         \
+    .get_speed = netdev_bsd_get_speed,               \
     .set_in4 = netdev_bsd_set_in4,                   \
     .get_addr_list = netdev_bsd_get_addr_list,       \
     .get_next_hop = netdev_bsd_get_next_hop,         \
@@ -1525,6 +1551,7 @@ destroy_tap(int fd, const char *name)
     struct ifreq ifr;
 
     close(fd);
+    memset(&ifr, 0, sizeof ifr);
     strcpy(ifr.ifr_name, name);
     /* XXX What to do if this call fails? */
     af_inet_ioctl(SIOCIFDESTROY, &ifr);
@@ -1536,6 +1563,7 @@ get_flags(const struct netdev *netdev, int *flags)
     struct ifreq ifr;
     int error;
 
+    memset(&ifr, 0, sizeof ifr);
     error = af_inet_ifreq_ioctl(netdev_get_kernel_name(netdev), &ifr,
                                 SIOCGIFFLAGS, "SIOCGIFFLAGS");
 
@@ -1549,6 +1577,7 @@ set_flags(const char *name, int flags)
 {
     struct ifreq ifr;
 
+    memset(&ifr, 0, sizeof ifr);
     ifr_set_flags(&ifr, flags);
 
     return af_inet_ifreq_ioctl(name, &ifr, SIOCSIFFLAGS, "SIOCSIFFLAGS");
