@@ -61,6 +61,27 @@ static struct vlog_rate_limit err_rl = VLOG_RATE_LIMIT_INIT(60, 5);
 uint16_t tnl_udp_port_min = 32768;
 uint16_t tnl_udp_port_max = 61000;
 
+ovs_be16
+netdev_tnl_get_src_port(struct dp_packet *packet)
+{
+    uint32_t hash;
+
+    if (OVS_LIKELY(dp_packet_rss_valid(packet))) {
+        hash = dp_packet_get_rss_hash(packet);
+    } else {
+        struct flow flow;
+
+        flow_extract(packet, &flow);
+        hash = flow_hash_5tuple(&flow, 0);
+
+        dp_packet_set_rss_hash(packet, hash);
+    }
+
+    hash = ((uint64_t) hash * (tnl_udp_port_max - tnl_udp_port_min)) >> 32;
+
+    return htons(hash + tnl_udp_port_min);
+}
+
 void *
 netdev_tnl_ip_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
                   unsigned int *hlen)
@@ -240,71 +261,31 @@ udp_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
     return udp + 1;
 }
 
-/* Calculate inner l2 l3 l4 len as tunnel outer header is not
- * encapsulated now. */
 static void
 dp_packet_tnl_ol_process(struct dp_packet *packet,
                          const struct ovs_action_push_tnl *data)
 {
-    struct udp_header *udp = NULL;
-    uint8_t opt_len = 0;
-    struct eth_header *eth = NULL;
     struct ip_header *ip = NULL;
-    struct genevehdr *gnh = NULL;
 
-    /* l2 l3 l4 len refer to inner len, tunnel outer
-     * header is not encapsulated here. */
     if (dp_packet_hwol_l4_mask(packet)) {
         ip = dp_packet_l3(packet);
 
-        if (ip->ip_proto == IPPROTO_TCP) {
-            struct tcp_header *th = dp_packet_l4(packet);
-            dp_packet_set_l4_len(packet, TCP_OFFSET(th->tcp_ctl) * 4);
-        } else if (ip->ip_proto == IPPROTO_UDP) {
-            dp_packet_set_l4_len(packet, UDP_HEADER_LEN);
-        } else if (ip->ip_proto == IPPROTO_SCTP) {
-            dp_packet_set_l4_len(packet, SCTP_HEADER_LEN);
-        }
-
-        dp_packet_set_l3_len(packet, (char *) dp_packet_l4(packet) -
-                                     (char *) dp_packet_l3(packet));
-
         if (data->tnl_type == OVS_VPORT_TYPE_GENEVE ||
             data->tnl_type == OVS_VPORT_TYPE_VXLAN) {
 
             if (IP_VER(ip->ip_ihl_ver) == 4) {
                 dp_packet_hwol_set_tx_ipv4(packet);
-                dp_packet_hwol_tx_ip_csum(packet);
+                dp_packet_hwol_set_tx_ip_csum(packet);
             } else if (IP_VER(ip->ip_ihl_ver) == 6) {
                 dp_packet_hwol_set_tx_ipv6(packet);
             }
         }
+    }
 
-        /* Attention please, tunnel inner l2 len is consist of udp header
-         * len and tunnel header len and inner l2 len. */
-        if (data->tnl_type == OVS_VPORT_TYPE_GENEVE) {
-            eth = (struct eth_header *)(data->header);
-            ip = (struct ip_header *)(eth + 1);
-            udp = (struct udp_header *)(ip + 1);
-            gnh = (struct genevehdr *)(udp + 1);
-            opt_len = gnh->opt_len * 4;
-            dp_packet_hwol_set_tunnel_geneve(packet);
-            dp_packet_set_l2_len(packet, (char *) dp_packet_l3(packet) -
-                                         (char *) dp_packet_eth(packet) +
-                                         GENEVE_BASE_HLEN + opt_len);
-        } else if (data->tnl_type == OVS_VPORT_TYPE_VXLAN) {
-            dp_packet_hwol_set_tunnel_vxlan(packet);
-            dp_packet_set_l2_len(packet, (char *) dp_packet_l3(packet) -
-                                         (char *) dp_packet_eth(packet) +
-                                         VXLAN_HLEN);
-        }
-    } else {
-        /* Mark non-l4 packets as tunneled. */
-        if (data->tnl_type == OVS_VPORT_TYPE_GENEVE) {
-            dp_packet_hwol_set_tunnel_geneve(packet);
-        } else if (data->tnl_type == OVS_VPORT_TYPE_VXLAN) {
-            dp_packet_hwol_set_tunnel_vxlan(packet);
-        }
+    if (data->tnl_type == OVS_VPORT_TYPE_GENEVE) {
+        dp_packet_hwol_set_tunnel_geneve(packet);
+    } else if (data->tnl_type == OVS_VPORT_TYPE_VXLAN) {
+        dp_packet_hwol_set_tunnel_vxlan(packet);
     }
 }
 
@@ -316,14 +297,18 @@ netdev_tnl_push_udp_header(const struct netdev *netdev OVS_UNUSED,
     uint16_t l3_ofs = packet->l3_ofs;
     uint16_t l4_ofs = packet->l4_ofs;
     struct udp_header *udp;
+    ovs_be16 udp_src;
     int ip_tot_size;
 
+    /* We may need to re-calculate the hash and this has to be done before
+     * modifying the packet. */
+    udp_src = netdev_tnl_get_src_port(packet);
+
     dp_packet_tnl_ol_process(packet, data);
     udp = netdev_tnl_push_ip_header(packet, data->header, data->header_len,
                                     &ip_tot_size, 0);
 
-    /* set udp src port */
-    udp->udp_src = netdev_tnl_get_src_port(packet);
+    udp->udp_src = udp_src;
     udp->udp_len = htons(ip_tot_size);
 
     if (udp->udp_csum) {
@@ -871,13 +856,18 @@ netdev_gtpu_push_header(const struct netdev *netdev,
     struct netdev_vport *dev = netdev_vport_cast(netdev);
     struct udp_header *udp;
     struct gtpuhdr *gtpuh;
+    ovs_be16 udp_src;
     int ip_tot_size;
     unsigned int payload_len;
 
+    /* We may need to re-calculate the hash and this has to be done before
+     * modifying the packet. */
+    udp_src = netdev_tnl_get_src_port(packet);
+
     payload_len = dp_packet_size(packet);
     udp = netdev_tnl_push_ip_header(packet, data->header, data->header_len,
                                     &ip_tot_size, 0);
-    udp->udp_src = netdev_tnl_get_src_port(packet);
+    udp->udp_src = udp_src;
     udp->udp_len = htons(ip_tot_size);
     /* Postpone checksum to the egress netdev. */
     dp_packet_hwol_set_csum_udp(packet);
@@ -932,9 +922,9 @@ netdev_srv6_build_header(const struct netdev *netdev,
                          const struct netdev_tnl_build_header_params *params)
 {
     const struct netdev_tunnel_config *tnl_cfg;
+    union ovs_16aligned_in6_addr *s;
     const struct in6_addr *segs;
     struct srv6_base_hdr *srh;
-    struct in6_addr *s;
     ovs_be16 dl_type;
     int nr_segs;
     int i;
@@ -978,8 +968,7 @@ netdev_srv6_build_header(const struct netdev *netdev,
         return EOPNOTSUPP;
     }
 
-    s = ALIGNED_CAST(struct in6_addr *,
-                     (char *) srh + sizeof *srh);
+    s = (union ovs_16aligned_in6_addr *) (srh + 1);
     for (i = 0; i < nr_segs; i++) {
         /* Segment list is written to the header in reverse order. */
         memcpy(s, &segs[nr_segs - i - 1], sizeof *s);
@@ -1068,7 +1057,10 @@ netdev_srv6_pop_header(struct dp_packet *packet)
     }
 
     pkt_metadata_init_tnl(md);
-    netdev_tnl_ip_extract_tnl_md(packet, tnl, &hlen);
+    if (!netdev_tnl_ip_extract_tnl_md(packet, tnl, &hlen)) {
+        goto err;
+    }
+
     dp_packet_reset_packet(packet, hlen);
 
     return packet;
