@@ -43,6 +43,10 @@ enum netdev_ol_flags {
     NETDEV_TX_OFFLOAD_UDP_CKSUM = 1 << 2,
     NETDEV_TX_OFFLOAD_SCTP_CKSUM = 1 << 3,
     NETDEV_TX_OFFLOAD_TCP_TSO = 1 << 4,
+    NETDEV_TX_VXLAN_TNL_TSO = 1 << 5,
+    NETDEV_TX_GENEVE_TNL_TSO = 1 << 6,
+    NETDEV_TX_OFFLOAD_OUTER_IP_CKSUM = 1 << 7,
+    NETDEV_TX_OFFLOAD_OUTER_UDP_CKSUM = 1 << 8,
 };
 
 /* A network device (e.g. an Ethernet device).
@@ -96,7 +100,8 @@ struct netdev {
 
     /* Functions to control flow offloading. */
     OVSRCU_TYPE(const struct netdev_flow_api *) flow_api;
-    struct netdev_hw_info hw_info;	/* offload-capable netdev info */
+    const char *dpif_type;          /* Type of dpif this netdev belongs to. */
+    struct netdev_hw_info hw_info;  /* Offload-capable netdev info. */
 };
 
 static inline void
@@ -499,6 +504,15 @@ struct netdev_class {
                         enum netdev_features *supported,
                         enum netdev_features *peer);
 
+    /* Stores the current and maximum supported link speed by 'netdev' into
+     * each of '*current' and '*max'. Each value represents the speed in Mbps.
+     * If any of the speeds is unknown, a zero value must be stored.
+     *
+     * This function may be set to null if it would always return EOPNOTSUPP.
+     */
+    int (*get_speed)(const struct netdev *netdev, uint32_t *current,
+                     uint32_t *max);
+
     /* Set the features advertised by 'netdev' to 'advertise', which is a
      * set of NETDEV_F_* bits.
      *
@@ -513,13 +527,16 @@ struct netdev_class {
      * NETDEV_PT_LEGACY_L2. */
     enum netdev_pt_mode (*get_pt_mode)(const struct netdev *netdev);
 
-    /* Attempts to set input rate limiting (policing) policy, such that up to
-     * 'kbits_rate' kbps of traffic is accepted, with a maximum accumulative
-     * burst size of 'kbits' kb.
+    /* Attempts to set input rate limiting (policing) policy, such that:
+     * - up to 'kbits_rate' kbps of traffic is accepted, with a maximum
+     *   accumulative burst size of 'kbits' kb; and
+     * - up to 'kpkts' kpps of traffic is accepted, with a maximum
+     *   accumulative burst size of 'kpkts' kilo packets.
      *
      * This function may be set to null if policing is not supported. */
     int (*set_policing)(struct netdev *netdev, unsigned int kbits_rate,
-                        unsigned int kbits_burst);
+                        unsigned int kbits_burst, unsigned int kpkts_rate,
+                        unsigned int kpkts_burst);
 
     /* Adds to 'types' all of the forms of QoS supported by 'netdev', or leaves
      * it empty if 'netdev' does not support QoS.  Any names added to 'types'
@@ -850,6 +867,7 @@ extern const struct netdev_class netdev_tap_class;
 
 #ifdef HAVE_AF_XDP
 extern const struct netdev_class netdev_afxdp_class;
+extern const struct netdev_class netdev_afxdp_nonpmd_class;
 #endif
 #ifdef  __cplusplus
 }
