@@ -32,7 +32,13 @@ bool ovs_router_lookup(uint32_t mark, const struct in6_addr *ip_dst,
 void ovs_router_init(void);
 void ovs_router_insert(uint32_t mark, const struct in6_addr *ip_dst,
                        uint8_t plen, bool local,
-                       const char output_bridge[], const struct in6_addr *gw);
+                       const char output_bridge[], const struct in6_addr *gw,
+                       const struct in6_addr *prefsrc);
+void ovs_router_force_insert(uint32_t mark, const struct in6_addr *ip_dst,
+                             uint8_t plen, bool local,
+                             const char output_bridge[],
+                             const struct in6_addr *gw,
+                             const struct in6_addr *prefsrc);
 void ovs_router_flush(void);
 
 void ovs_router_disable_system_routing_table(void);
