@@ -216,7 +216,7 @@ int
 stream_open(const char *name, struct stream **streamp, uint8_t dscp)
 {
     const struct stream_class *class;
-    struct stream *stream;
+    struct stream *stream = NULL;
     char *suffix_copy;
     int error;
 
@@ -547,7 +547,7 @@ int
 pstream_open(const char *name, struct pstream **pstreamp, uint8_t dscp)
 {
     const struct pstream_class *class;
-    struct pstream *pstream;
+    struct pstream *pstream = NULL;
     char *suffix_copy;
     int error;
 
@@ -788,7 +788,7 @@ stream_parse_target_with_default_port(const char *target, int default_port,
                                       struct sockaddr_storage *ss)
 {
     return ((!strncmp(target, "tcp:", 4) || !strncmp(target, "ssl:", 4))
-            && inet_parse_active(target + 4, default_port, ss, true));
+            && inet_parse_active(target + 4, default_port, ss, true, NULL));
 }
 
 /* Attempts to guess the content type of a stream whose first few bytes were
